#!/bin/bash

# Variables
LOG_OUT="okteto_upgrader.log"
DEPENDENCIES=(jq kubectl helm)
BLUE="\033[0;34m"
GREEN="\033[32m"
RED="\033[31m"
YELLOY="\033[0;33m"
RESET="\033[0m"

# Define Flags
help(){
  printf " HELP\n\n"
  printf "\tDescription:\tThis Bash tool is used to upgrade Okteto Enterprise Version.\n"
  printf "\t\t\tTo work correctly, the next dependencies must be satisfied: ${DEPENDENCIES[*]}\n\n"
  printf "\tOptions:\t-f\tDefine a new config file.\n"
  printf "\t\t\t-h\tShow this help.\n"
  printf "\t\t\t-l\tDefine a custom log name.\n"
  printf "\t\t\t-L\tDisable log.\n"
  exit 0
}

while getopts Lhl:f: flag
do
  case $flag in
    f)  HELM_CONFIG="$OPTARG" ;;
    h)  help ;;
    L)  LOG_OUT="/dev/null" ;;
    l)  LOG_OUT="$OPTARG" ;;
    *)  exit 1 ;;
  esac
done

# Logs and notify.
date_format(){
  date '+%Y/%m/%d %H:%M:%S'
}
log(){
  case $1 in
    s) OUT_TYPE="$(date_format) [$GREEN"SUCCESS"$RESET] $2" ;;
    i) OUT_TYPE="$(date_format) [$BLUE"INFO"$RESET] $2" ;;
    w) OUT_TYPE="$(date_format) [$YELLOW"WARNING"$RESET] $2" ;;
    e) OUT_TYPE="$(date_format) [$RED"ERROR"$RESET] $2" ;;
  esac
  echo -e "$OUT_TYPE" | tee -a $LOG_OUT &>/dev/null
}

# Action if Kubectl/helm failed
kube_success(){
  if [ $? -eq 0 ]; then
    log i "$1"
  else 
    log e "$1 failed, Make sure you have access and permissions in the Kubernetes Cluster."
    printf "$RED $1 failed, Make sure you have access and permissions in the Kubernetes Cluster.$RESET\n"
    exit 1
  fi 
}

# Check Dependencies
for i in "${DEPENDENCIES[@]}"; do
  which $i &> /dev/null
  if [ $? -eq 0 ]; then
    log i "$i Dependency satisfied"
  else 
    log e "$1 not found."
    printf "$RED $i is necessary to run this script. You have to install it. $RESET\n"
    exit 1
  fi
done 

# Get the release name.
log i "Getting release name"
RELEASE_NAME=$(helm list -n okteto | grep "okteto-enterprise-" |awk '{print $1}')
kube_success "Get Release Name"

# If not set a new config.yaml get the release config.
if [ -z "${HELM_CONFIG}" ]; then 
  log i "Getting release config"
  HELM_CONFIG=config.yaml
  helm get values -n okteto $RELEASE_NAME | grep -v "USER-SUPPLIED VALUES:" > $HELM_CONFIG
  kube_success "Get Config"
fi

# Delete Certificates
CERTIFICATES=(default-ssl-certificate $RELEASE_NAME-okteto-enterprise-ca $RELEASE_NAME-okteto-enterprise-internal-tls)
for i in "${CERTIFICATES[@]}"; do
  log i "Deleting $i Certificate"
  kubectl delete certificate -n okteto $i &> /dev/null
  kube_success "Delete certificate $i"
done

# Delete Issuers
ISSUERS=($RELEASE_NAME-okteto-enterprise $RELEASE_NAME-okteto-enterprise-ca $RELEASE_NAME-okteto-enterprise-selfsign)
for i in "${ISSUERS[@]}"; do
  log i "Deleting $i Issuer"
  kubectl delete issuer -n okteto $i &> /dev/null
  kube_success "Delete issuer $i"
done 

# Delete the Cert Manager deployments
CERT_MANAGER_DEPLOYMENTS=(cert-manager cert-manager-cainjector cert-manager-webhook)
for i in "${CERT_MANAGER_DEPLOYMENTS[@]}";do
  log i "deleting $i deployment"
  kubectl delete deployment $RELEASE_NAME-$i -n okteto  &> /dev/null
  if [ $? -eq 0 ]; then
    log i "$RELEASE_NAME-$i deleted"
  else 
    log w "$RELEASE_NAME-$i can´t be deleted. It probably doesn't exist."
  fi
done

# Delete the Cert Manager Mutating and Validating Webhooks
WEBHOOKS=(mutatingwebhookconfigurations validatingwebhookconfigurations)
for i in "${WEBHOOKS[@]}"; do
  log i "Delenting $RELEASE_NAME-cert-manager-webhook $1"
  kubectl delete $i $RELEASE_NAME-cert-manager-webhook &> /dev/null
  if [ $? -eq 0 ]; then
  log i "$RELEASE_NAME-cert-manager-webhook $1 deleted"
  else
  log w "$RELEASE_NAME-cert-manager-webhook $1 can\`t be delete. If the old version had Cert-Manager installed it may not work correctly. The update should correct the problem."
  fi
done

# Get the Kubernetes version to set the crds.
log i "Getting Kubernetes Version"
KUBE_VERSION=$(kubectl version -o json |jq -r ".serverVersion.minor" | tr -d -c 0-9=)
log i "Get kubernetes version"

if [ $KUBE_VERSION -lt "15" ];then
CRDS="crds-legacy.yaml"
else
CRDS="crds.yaml"
fi

# Upgrade the CRDs
log i "Upgrading $CRDS"
kubectl apply --validate=false -f https://charts.okteto.com/$CRDS &> /dev/null
kube_success "Update $CRDS"

log i "Ugrading Helm chart"
helm upgrade -n okteto $RELEASE_NAME -f $HELM_CONFIG okteto/okteto-enterprise  &> /dev/null
kube_success "Chart upgraded"
printf "$GREEN> Okteto Enterprise Upgraded!\n$RESET"
log s "Okteto Enterprise Upgraded!"
